<?php

declare(strict_types=1);

namespace Drupal\Tests\otlog\Constraint;

use Drupal\Tests\otlog\IntrospectableLogRecord;
use PHPUnit\Framework\Constraint\Constraint;

/**
 * Log record body constraint.
 */
final class LogRecordConstraint extends Constraint {

  public function __construct(
    protected readonly ?string $body = NULL,
    protected readonly ?int $timestamp = NULL,
    protected readonly ?int $observedTimestamp = NULL,
    protected readonly ?int $severityNumber = NULL,
    protected readonly ?string $severityText = NULL,
    protected readonly ?array $attributes = NULL,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function toString(): string {
    return 'LogRecord has specified properties';
  }

  /**
   * {@inheritdoc}
   */
  protected function matches(mixed $other): bool {
    $introspectable = new IntrospectableLogRecord($other);
    if (isset($this->body) && $introspectable->getBody() !== $this->body) {
      return FALSE;
    }

    if (isset($this->timestamp) && $introspectable->getTimestamp() !== $this->timestamp) {
      return FALSE;
    }

    if (isset($this->observedTimestamp) && $introspectable->getTimestamp() !== $this->observedTimestamp) {
      return FALSE;
    }

    if (isset($this->severityNumber) && $introspectable->getSeverityNumber() !== $this->severityNumber) {
      return FALSE;
    }

    if (isset($this->severityText) && $introspectable->getSeverityText() !== $this->severityText) {
      return FALSE;
    }

    if (isset($this->attributes)) {
      $actual = $introspectable->getAttributes();
      foreach ($this->attributes as $key => $value) {
        if (!isset($actual[$key])) {
          return FALSE;
        }
        if ($actual[$key] !== $value) {
          return FALSE;
        }
      }
    }

    return TRUE;
  }

}
